@@ -27,7 +27,6 @@
 #include <libintl.h>
 
 #include <gtkmm.h>
-#include <libglademm.h>
 
 #include <pulse/pulseaudio.h>
 
@@ -35,6 +34,13 @@
 #define GLADE_FILE "pavucontrol.glade"
 #endif
 
+/* Can be removed when PulseAudio 0.9.23 or newer is required */
+#ifndef PA_VOLUME_UI_MAX
+# define PA_VOLUME_UI_MAX (pa_sw_volume_from_dB(+11.0))
+#endif
+
+#define HAVE_SOURCE_OUTPUT_VOLUMES PA_CHECK_VERSION(0,99,0)
+
 enum SinkInputType {
     SINK_INPUT_ALL,
     SINK_INPUT_CLIENT,
